@@ -23,6 +23,7 @@ MainWindow::MainWindow(Window parent, bool user) :
     ,m_language1VariantModel(new QStandardItemModel(this))
     ,m_language2VariantModel(new QStandardItemModel(this))
     ,m_layoutModel(new QStandardItemModel(this))
+    ,m_keyboardModel(new QStandardItemModel(this))
     ,m_languageSwitchModel(new QStandardItemModel(this))
     ,m_baseXMLParser(new BaseXMLParser(this))
     ,ui(new Ui::MainWindow)
@@ -59,6 +60,7 @@ MainWindow::~MainWindow()
     delete m_language2VariantModel;
     delete m_layoutModel;
     delete m_languageSwitchModel;
+    delete m_keyboardModel;
 }
 
 void MainWindow::setLayoutModel()
@@ -167,24 +169,24 @@ void MainWindow::setKbdModelVB(const QString & kbdModel)
     HalKeyboards halKeyboards(this);
     QBTKeyboards btkeyboards(this);
 
-    QStandardItemModel *model = new QStandardItemModel(this);
+    m_keyboardModel->clear();
 
     for (int i = 0; i < halKeyboards.count(); ++i)
     {
         QString iconName=btkeyboards.value(halKeyboards.keys().value(i)).isEmpty()?
                     "control_keyboard":"control_bluetooth_uncategorised";
-        model->appendRow(new QStandardItem( QIcon::fromTheme(iconName), halKeyboards.keys().value(i) ));
+	m_keyboardModel->appendRow(new QStandardItem( QIcon::fromTheme(iconName), halKeyboards.keys().value(i) ));
     }
 
     for(int i = 0; i < m_halFdiProducts.count(); i++)
     {
         QString fromFdi=m_halFdiProducts.keys().value(i);
         if(halKeyboards.keys().indexOf(fromFdi) == -1)
-            model->appendRow(new QStandardItem( QIcon::fromTheme("control_keyboard"), fromFdi ));
+	    m_keyboardModel->appendRow(new QStandardItem( QIcon::fromTheme("control_keyboard"), fromFdi ));
 
     }
     QMaemo5ListPickSelector * listpicker = new QMaemo5ListPickSelector();
-    listpicker->setModel(model);
+    listpicker->setModel(m_keyboardModel);
 
     ui->vbExternaKeyboard->setIcon(QIcon::fromTheme("control_keyboard"));
     ui->vbExternaKeyboard->setValueLayout(QMaemo5ValueButton::ValueUnderText);
@@ -484,84 +486,20 @@ void MainWindow::makeFdi()
         QString options=kbdProps[5]+",caps:shiftlock";
 
 	file.write(QString(FDI_PRODUCT).arg(product).arg(kbdProps[0]).arg(layouts).arg(options).toLatin1());
-	setXkbMap(product,kbdProps[0],layouts,options);
 
     }
     file.write(FDI_TRLR);
 }
-void MainWindow::setInternalXkbMap()
+
+#define TMP_HAL_FILE "/tmp/extkbd.lshal.tmp"
+void MainWindow::applyKbdMap()
 {
 
-    FILE * fp=popen(QString("hildon-im-xkbtool --list | grep \"omap_twl4030keypad\"").toLatin1(),"r");
-    if(fp)
+    HalKeyboards kbds(this);
+    for(int i=0;i<kbds.count();i++)
     {
-	QTextStream device_stream( fp, QIODevice::ReadOnly );
-	QString device = device_stream.readLine();
-
-	if((!device.isNull()) && (!device.isEmpty()))
-	{
-	    QString int_kb_model=QGtkStyle::getGConfString("/apps/osso/inputmethod/int_kb_model");
-	    QString int_kb_layout=QGtkStyle::getGConfString("/apps/osso/inputmethod/int_kb_layout");
-	    QString device_id=device.split(",").value(0).split(" ").value(1);
-	    qDebug()<<QString("setxkbmap -device \"%1\" -model \"%2\" -layout \"%3\"").arg(device_id).arg(int_kb_model).arg(int_kb_layout).toLatin1();
-	    system(QString("setxkbmap -device \"%1\" -model \"%2\" -layout \"%3\" -synch").arg(device_id).arg(int_kb_model).arg(int_kb_layout).toLatin1());
-	}
-	else
-	{
-	    QMaemo5InformationBox::information(this, QString("omap_twl4030keypad not found, is this N900?"),
-						QMaemo5InformationBox::NoTimeout);
-	    return;
-	}
+	system(QString("sudo /usr/libexec/extkbd-set %1").arg(kbds.values().value(i)).toLatin1());
     }
-    else
-	QMaemo5InformationBox::information(this, QString("hildon-im-xkbtool error %1<br>%2").arg(errno).arg(strerror(errno)),
-					    QMaemo5InformationBox::NoTimeout);
-    QApplication::processEvents();
-    //system("setxkbmap");
-    //QApplication::processEvents();
-
-
-}
-
-bool MainWindow::setXkbMap(const QString & product,const QString & model,const QString & layout, const QString & options)
-{
-    FILE * fp=popen(QString("hildon-im-xkbtool --list | grep \"%1\"").arg(product).toLatin1(),"r");
-    if(fp)
-    {
-	QString device;
-	QTextStream device_stream( fp, QIODevice::ReadOnly );
-	while((!(device = device_stream.readLine()).isNull()) && (!device.isEmpty()))
-	{
-	    QString device_id=device.split(",").value(0).split(" ").value(1);
-
-	    system(QString("setxkbmap "
-			   "-device \"%1\" "
-			   "-model  \"%2\" "
-			   "-layout \"%3\" "
-			   "-option \"%4\" "
-			   "-synch")
-			.arg(device_id)
-			.arg(model)
-			.arg(layout)
-			.arg(options)
-			.toLatin1()
-		   );
-	    QApplication::processEvents();
-
-	    system("setxkbmap");
-	}
-
-	QApplication::processEvents();
-
-	pclose(fp);
-    }
-    else
-    {
-	QMaemo5InformationBox::information(this, QString("hildon-im-xkbtool error %1<br>%2").arg(errno).arg(strerror(errno)),
-						QMaemo5InformationBox::NoTimeout);
-	return false;
-    }
-    return true;
 }
 
 void MainWindow::on_buttonBox_accepted()
@@ -575,8 +513,8 @@ void MainWindow::on_buttonBox_accepted()
     QApplication::processEvents();
     system("sudo /usr/libexec/extkbd-set");
     QApplication::processEvents();
-    setInternalXkbMap();
 
+    applyKbdMap();
     accept();
 }
 
@@ -584,5 +522,6 @@ void MainWindow::on_pushButton_clicked()
 {
     ToolsDialog dlg(this);
     dlg.exec();
+    setKbdModelVB();
 }
 
