@@ -46,8 +46,7 @@ protected:
     void saveCurrentKeyboard();
 
     void makeFdi();
-    bool setXkbMap(const QString & product,const QString & model,const QString & layout, const QString & options);
-    void setInternalXkbMap();
+    void applyKbdMap();
 
     //contains keyboards found in "/usr/share/hal/fdi/policy/20thirdparty/99-x11-external-keyboard.fdi"
     //key - product id(i.e. keyboard name), - value [key- key_value, value - attribute text,...]
@@ -60,6 +59,7 @@ protected:
     QStandardItemModel * m_language2VariantModel;
     QStandardItemModel * m_descriptionModel;
     QStandardItemModel * m_layoutModel;
+    QStandardItemModel * m_keyboardModel;
     QStandardItemModel * m_languageSwitchModel;
     BaseXMLParser * m_baseXMLParser;
 
